import mimetypes, json, requests, base64, hmac, hashlib, time

from django.conf import settings
from django.utils import timezone
from django.db import models
from django.core.files import File

from filepicker import FilepickerFile

from utils import store_from_upload_file, store_from_url

try:
    EXPIRES_IN_SECONDS = settings.FILEPICKER_EXPIRES_IN_SECONDS
except:
    EXPIRES_IN_SECONDS = 60*60

EXPIRY = int(time.time() + EXPIRES_IN_SECONDS)

class FilepickerField(models.URLField):
    def __init__(self, verbose_name=None, name=None, **kwargs):
        kwargs['max_length'] = kwargs.get('max_length', 2000)
        super(models.URLField, self).__init__(verbose_name, name, **kwargs)

    def from_db_value(self, value, expression, connection, context):
        if value is None:
            return value
        return get_signed_url(value)

    def to_python(self, value):
        return value

class UrlSignature(models.Model):
    file_handle = models.CharField(max_length=255)
    policy = models.CharField(max_length=255)
    signature = models.CharField(max_length=255)
    expires = models.DateTimeField(null=True, blank=True)

class FilepickerModel(models.Model):
    def save(self, *args, **kwargs):
        if not self.id:
            for field in self._meta.fields:
                if isinstance(field, FilepickerField):
                    if self.__getattribute__(field.name):
                        if isinstance(self.__getattribute__(field.name), File):
                            self.__setattr__(field.name, store_from_upload_file(self.__getattribute__(field.name)))
                        else:
                            self.__setattr__(field.name, store_from_url(self.__getattribute__(field.name)))
        super(FilepickerModel, self).save(*args, **kwargs)

    class Meta:
        abstract = True

def get_signed_url(url):
    if url is None or url == '':
        return

    now = timezone.now()

    fpfile = FilepickerFile(url=url)
    # remove existing query params
    base_url = fpfile.url.split('?')[0]

    sig = UrlSignature.objects.filter(file_handle=fpfile.handle, expires__gte=now).order_by('-expires').first()
    if sig:
        return '{0}?policy={1}&signature={2}'.format(base_url, sig.policy, sig.signature)

    expiry = str(EXPIRY)
    json_policy = '{"handle":"%s","expiry":%s}' % (fpfile.handle, expiry)

    policy = base64.urlsafe_b64encode(json_policy)
    signature = hmac.new(settings.FILEPICKER_APP_SECRET, policy, hashlib.sha256).hexdigest()

    new_sig = UrlSignature(file_handle=fpfile.handle, policy=policy, signature=signature, expires=now + timezone.timedelta(0, EXPIRES_IN_SECONDS))
    new_sig.save()

    return '{0}?policy={1}&signature={2}'.format(base_url, policy, signature)



from setuptools import setup, find_packages

setup(
    name='django-filepicker-model',
    version='0.1.1',
    description='Filepicker for Django without a frontend',
    author='jpwagner',
    author_email='jim@sevenleague.co',
    url='http://github.com/sevenleague/django-filepicker-model',
    packages=find_packages(),
    install_requires=['django >= 1.3','requests', 'filepicker'],
    zip_safe=False,
)


import mimetypes, json, requests, base64, hmac, hashlib, time

from django.conf import settings
from django.utils import timezone
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

from filepicker import FilepickerClient, FilepickerFile

try:
    EXPIRES_IN_SECONDS = settings.FILEPICKER_EXPIRES_IN_SECONDS
except:
    EXPIRES_IN_SECONDS = 60*60

EXPIRY = int(time.time() + EXPIRES_IN_SECONDS)


class CustomFPClient(FilepickerClient):
    def store_from_upload_file(self, file, filename, storage=None, policy_name=None, **kwargs):
        mimetype = mimetypes.guess_type(filename)
        files = {'fileUpload': (filename, file.read(), mimetype)}
        params = {}
        if policy_name:
            params.update(self.policies[policy_name].signature_params())
        if kwargs:
            params.update(kwargs)
        return self.__post(storage, files=files, params=params)

    def __post(self, storage, data=None, files=None, params=None):
        storage = storage or self.storage
        post_url = '{}/store/{}'.format(self.API_URL, storage)
        params['key'] = self.api_key

        response = requests.post(post_url, data=data, files=files,
                                 params=params, headers=self.HEADERS)
        try:
            response_dict = json.loads(response.text)
            return FilepickerFile(response_dict=response_dict,
                                  api_key=self.api_key,
                                  app_secret=self.app_secret,
                                  policies=self.policies)
        except ValueError:
            return response.raise_for_status()

def store_from_upload_file(file):
    client = CustomFPClient(api_key=settings.FILEPICKER_API_KEY, app_secret=settings.FILEPICKER_APP_SECRET)
    expiry = str(EXPIRY)
    client.add_policy(name='allow_storing',
                      policy={
                               'call': 'store',
                               'expiry': expiry
                              })

    response = client.store_from_upload_file(file, file.name, policy_name='allow_storing')
    if isinstance(response, FilepickerFile):
        return '{0}{1}'.format(response.FILE_API_URL, response.handle)
    return response

def store_from_url(url):
    client = CustomFPClient(api_key=settings.FILEPICKER_API_KEY, app_secret=settings.FILEPICKER_APP_SECRET)
    expiry = str(EXPIRY)
    client.add_policy(name='allow_storing',
                      policy={
                               'call': 'store',
                               'expiry': expiry
                              })

    response = client.store_from_url(url, policy_name='allow_storing')
    if isinstance(response, FilepickerFile):
        return '{0}{1}'.format(response.FILE_API_URL, response.handle)
    return response
